@@ -1,50 +1,8 @@
-start_server {tags {"pubsub"}} {
-    proc __consume_subscribe_messages {client type channels} {
-        set numsub -1
-        set counts {}
-
-        for {set i [llength $channels]} {$i > 0} {incr i -1} {
-            set msg [$client read]
-            assert_equal $type [lindex $msg 0]
-
-            # when receiving subscribe messages the channels names
-            # are ordered. when receiving unsubscribe messages
-            # they are unordered
-            set idx [lsearch -exact $channels [lindex $msg 1]]
-            if {[string match "*unsubscribe" $type]} {
-                assert {$idx >= 0}
-            } else {
-                assert {$idx == 0}
-            }
-            set channels [lreplace $channels $idx $idx]
-
-            # aggregate the subscription count to return to the caller
-            lappend counts [lindex $msg 2]
-        }
-
-        # we should have received messages for channels
-        assert {[llength $channels] == 0}
-        return $counts
-    }
-
-    proc subscribe {client channels} {
-        $client subscribe {*}$channels
-        __consume_subscribe_messages $client subscribe $channels
-    }
-
-    proc unsubscribe {client {channels {}}} {
-        $client unsubscribe {*}$channels
-        __consume_subscribe_messages $client unsubscribe $channels
-    }
-
-    proc psubscribe {client channels} {
-        $client psubscribe {*}$channels
-        __consume_subscribe_messages $client psubscribe $channels
-    }
-
-    proc punsubscribe {client {channels {}}} {
-        $client punsubscribe {*}$channels
-        __consume_subscribe_messages $client punsubscribe $channels
+start_server {tags {"pubsub network"}} {
+    if {$::singledb} {
+        set db 0
+    } else {
+        set db 9
     }
 
     test "Pub/Sub PING" {
@@ -200,6 +158,26 @@ start_server {tags {"pubsub"}} {
         r pubsub numsub abc def
     } {abc 0 def 0}
 
+    test "NUMPATs returns the number of unique patterns" {
+        set rd1 [redis_deferring_client]
+        set rd2 [redis_deferring_client]
+
+        # Three unique patterns and one that overlaps
+        psubscribe $rd1 "foo*"
+        psubscribe $rd2 "foo*"
+        psubscribe $rd1 "bar*"
+        psubscribe $rd2 "baz*"
+
+        set patterns [r pubsub numpat]
+
+        # clean up clients
+        punsubscribe $rd1
+        punsubscribe $rd2
+        assert_equal 3 $patterns
+        $rd1 close
+        $rd2 close
+    }
+
     test "Mix SUBSCRIBE and PSUBSCRIBE" {
         set rd1 [redis_deferring_client]
         assert_equal {1} [subscribe $rd1 {foo.bar}]
@@ -230,7 +208,7 @@ start_server {tags {"pubsub"}} {
         set rd1 [redis_deferring_client]
         assert_equal {1} [psubscribe $rd1 *]
         r set foo bar
-        assert_equal {pmessage * __keyspace@9__:foo set} [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:foo set" [$rd1 read]
         $rd1 close
     }
 
@@ -239,7 +217,7 @@ start_server {tags {"pubsub"}} {
         set rd1 [redis_deferring_client]
         assert_equal {1} [psubscribe $rd1 *]
         r set foo bar
-        assert_equal {pmessage * __keyevent@9__:set foo} [$rd1 read]
+        assert_equal "pmessage * __keyevent@${db}__:set foo" [$rd1 read]
         $rd1 close
     }
 
@@ -248,8 +226,8 @@ start_server {tags {"pubsub"}} {
         set rd1 [redis_deferring_client]
         assert_equal {1} [psubscribe $rd1 *]
         r set foo bar
-        assert_equal {pmessage * __keyspace@9__:foo set} [$rd1 read]
-        assert_equal {pmessage * __keyevent@9__:set foo} [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:foo set" [$rd1 read]
+        assert_equal "pmessage * __keyevent@${db}__:set foo" [$rd1 read]
         $rd1 close
     }
 
@@ -261,8 +239,8 @@ start_server {tags {"pubsub"}} {
         r set foo bar
         r lpush mylist a
         # No notification for set, because only list commands are enabled.
-        assert_equal {pmessage * __keyspace@9__:mylist lpush} [$rd1 read]
-        assert_equal {pmessage * __keyevent@9__:lpush mylist} [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:mylist lpush" [$rd1 read]
+        assert_equal "pmessage * __keyevent@${db}__:lpush mylist" [$rd1 read]
         $rd1 close
     }
 
@@ -273,10 +251,10 @@ start_server {tags {"pubsub"}} {
         r set foo bar
         r expire foo 1
         r del foo
-        assert_equal {pmessage * __keyspace@9__:foo expire} [$rd1 read]
-        assert_equal {pmessage * __keyevent@9__:expire foo} [$rd1 read]
-        assert_equal {pmessage * __keyspace@9__:foo del} [$rd1 read]
-        assert_equal {pmessage * __keyevent@9__:del foo} [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:foo expire" [$rd1 read]
+        assert_equal "pmessage * __keyevent@${db}__:expire foo" [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:foo del" [$rd1 read]
+        assert_equal "pmessage * __keyevent@${db}__:del foo" [$rd1 read]
         $rd1 close
     }
 
@@ -288,12 +266,12 @@ start_server {tags {"pubsub"}} {
         r lpush mylist a
         r rpush mylist a
         r rpop mylist
-        assert_equal {pmessage * __keyspace@9__:mylist lpush} [$rd1 read]
-        assert_equal {pmessage * __keyevent@9__:lpush mylist} [$rd1 read]
-        assert_equal {pmessage * __keyspace@9__:mylist rpush} [$rd1 read]
-        assert_equal {pmessage * __keyevent@9__:rpush mylist} [$rd1 read]
-        assert_equal {pmessage * __keyspace@9__:mylist rpop} [$rd1 read]
-        assert_equal {pmessage * __keyevent@9__:rpop mylist} [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:mylist lpush" [$rd1 read]
+        assert_equal "pmessage * __keyevent@${db}__:lpush mylist" [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:mylist rpush" [$rd1 read]
+        assert_equal "pmessage * __keyevent@${db}__:rpush mylist" [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:mylist rpop" [$rd1 read]
+        assert_equal "pmessage * __keyevent@${db}__:rpop mylist" [$rd1 read]
         $rd1 close
     }
 
@@ -306,9 +284,9 @@ start_server {tags {"pubsub"}} {
         r srem myset x
         r sadd myset x y z
         r srem myset x
-        assert_equal {pmessage * __keyspace@9__:myset sadd} [$rd1 read]
-        assert_equal {pmessage * __keyspace@9__:myset sadd} [$rd1 read]
-        assert_equal {pmessage * __keyspace@9__:myset srem} [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:myset sadd" [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:myset sadd" [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:myset srem" [$rd1 read]
         $rd1 close
     }
 
@@ -321,9 +299,9 @@ start_server {tags {"pubsub"}} {
         r zrem myzset x
         r zadd myzset 3 x 4 y 5 z
         r zrem myzset x
-        assert_equal {pmessage * __keyspace@9__:myzset zadd} [$rd1 read]
-        assert_equal {pmessage * __keyspace@9__:myzset zadd} [$rd1 read]
-        assert_equal {pmessage * __keyspace@9__:myzset zrem} [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:myzset zadd" [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:myzset zadd" [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:myzset zrem" [$rd1 read]
         $rd1 close
     }
 
@@ -334,8 +312,32 @@ start_server {tags {"pubsub"}} {
         assert_equal {1} [psubscribe $rd1 *]
         r hmset myhash yes 1 no 0
         r hincrby myhash yes 10
-        assert_equal {pmessage * __keyspace@9__:myhash hset} [$rd1 read]
-        assert_equal {pmessage * __keyspace@9__:myhash hincrby} [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:myhash hset" [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:myhash hincrby" [$rd1 read]
+        $rd1 close
+    }
+
+    test "Keyspace notifications: stream events test" {
+        r config set notify-keyspace-events Kt
+        r del mystream
+        set rd1 [redis_deferring_client]
+        assert_equal {1} [psubscribe $rd1 *]
+        r xgroup create mystream mygroup $ mkstream
+        r xgroup createconsumer mystream mygroup Bob
+        set id [r xadd mystream 1 field1 A]
+        r xreadgroup group mygroup Alice STREAMS mystream >
+        r xclaim mystream mygroup Mike 0 $id force
+        # Not notify because of "Lee" not exists.
+        r xgroup delconsumer mystream mygroup Lee
+        # Not notify because of "Bob" exists.
+        r xautoclaim mystream mygroup Bob 0 $id
+        r xgroup delconsumer mystream mygroup Bob
+        assert_equal "pmessage * __keyspace@${db}__:mystream xgroup-create" [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:mystream xgroup-createconsumer" [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:mystream xadd" [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:mystream xgroup-createconsumer" [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:mystream xgroup-createconsumer" [$rd1 read]
+        assert_equal "pmessage * __keyspace@${db}__:mystream xgroup-delconsumer" [$rd1 read]
         $rd1 close
     }
 
@@ -350,7 +352,7 @@ start_server {tags {"pubsub"}} {
         } else {
             fail "Key does not expire?!"
         }
-        assert_equal {pmessage * __keyevent@9__:expired foo} [$rd1 read]
+        assert_equal "pmessage * __keyevent@${db}__:expired foo" [$rd1 read]
         $rd1 close
     }
 
@@ -360,7 +362,7 @@ start_server {tags {"pubsub"}} {
         set rd1 [redis_deferring_client]
         assert_equal {1} [psubscribe $rd1 *]
         r psetex foo 100 1
-        assert_equal {pmessage * __keyevent@9__:expired foo} [$rd1 read]
+        assert_equal "pmessage * __keyevent@${db}__:expired foo" [$rd1 read]
         $rd1 close
     }
 
@@ -372,10 +374,11 @@ start_server {tags {"pubsub"}} {
         assert_equal {1} [psubscribe $rd1 *]
         r set foo bar
         r config set maxmemory 1
-        assert_equal {pmessage * __keyevent@9__:evicted foo} [$rd1 read]
+        assert_equal "pmessage * __keyevent@${db}__:evicted foo" [$rd1 read]
         r config set maxmemory 0
         $rd1 close
-    }
+        r config set maxmemory-policy noeviction
+    } {OK} {needs:config-maxmemory}
 
     test "Keyspace notifications: test CONFIG GET/SET of event flags" {
         r config set notify-keyspace-events gKE
@@ -387,4 +390,17 @@ start_server {tags {"pubsub"}} {
         r config set notify-keyspace-events EA
         assert_equal {AE} [lindex [r config get notify-keyspace-events] 1]
     }
+
+    test "Keyspace notifications: new key test" {
+        r config set notify-keyspace-events En
+        set rd1 [redis_deferring_client]
+        assert_equal {1} [psubscribe $rd1 *]
+        r set foo bar
+        # second set of foo should not cause a 'new' event
+        r set foo baz 
+        r set bar bar
+        assert_equal "pmessage * __keyevent@${db}__:new foo" [$rd1 read]
+        assert_equal "pmessage * __keyevent@${db}__:new bar" [$rd1 read]
+        $rd1 close
+    }
 }
